__version__ = "1.2.0.dev5"
"""
To update the version, use [hatch version](https://hatch.pypa.io/latest/version/#updating)

show current version:

```console
$ hatch version
1.0.0a0
```

update patch and beta version from `1.0.0a0` to `1.0.1b0`

```console
$ hatch version patch,beta
Old: 1.0.0a0
New: 1.0.1b0
```
"""
